<?php

/**
 * Interface FeaturizeInterface
 */
interface FeaturizeInterface {
  public function getComponents($feature);

  public function getDirectory($feature);

  public function getFeatures();

  public function getPackage($feature);

  public function getVersion($feature);
}

/**
 * Class FeaturizeBase
 */
abstract class FeaturizeBase implements FeaturizeInterface {
  protected $components = array(
    'dependencies' => array(),
    'features' => array(),
  );

  /**
   * @param $feature
   *
   * @return string
   */
  public function getPackage($feature) {
    return 'Features';
  }

  /**
   * @param $feature
   *
   * @return string
   */
  public function getVersion($feature) {
    return '7.x-1.0';
  }

  /**
   * @param $feature
   *
   * @return string
   */
  public function getDirectory($feature) {
    return conf_path() . '/modules/featurize/' . $feature;
  }

  /**
   * @param $module
   */
  public function addDependency($module) {
    $this->components['dependencies']['dependencies'][$module] = $module;
  }

  /**
   * @param $module
   */
  public function removeDependency($module) {
    unset($this->components['dependencies']['dependencies'][$module]);
  }

  /**
   * @param $component
   * @param $name
   */
  public function addExclusion($component, $name) {
    $this->components['dependencies']['features_exclude'][$component][$name] = $name;
  }

  /**
   * @param $component
   * @param null $name
   */
  public function removeExclusion($component, $name = NULL) {
    if (is_null($name)) {
      unset($this->components['dependencies']['features_exclude'][$component]);
    }
    else {
      unset($this->components['dependencies']['features_exclude'][$component][$name]);
    }
  }

  /**
   * @param $component
   * @param $name
   * @param bool $include
   */
  protected function setComponent($component, $name, $include = TRUE) {
    if ($export_options = featurize_get_export_options($component)) {
      if (isset($export_options[$name])) {
        if ($include) {
          $this->components['features'][$component][$name] = $name;
        }
        else {
          unset($this->components['features'][$component][$name]);
        }
      }
    }
  }

  /**
   * @param $component
   * @param $name
   */
  public function addComponent($component, $name) {
    $this->setComponent($component, $name, TRUE);
  }

  /**
   * @param $component
   * @param $name
   */
  public function removeComponent($component, $name) {
    $this->setComponent($component, $name, FALSE);
  }

  /**
   * @param $component
   * @param array $names
   * @param bool $include
   */
  protected function setComponents($component, array $names, $include = TRUE) {
    foreach ($names as $name) {
      $this->setComponent($component, $name, $include);
    }
  }

  /**
   * @param $component
   * @param array $names
   */
  public function addComponents($component, array $names) {
    $this->setComponents($component, $names, TRUE);
  }

  /**
   * @param $component
   * @param array $names
   */
  public function removeComponents($component, array $names) {
    $this->setComponents($component, $names, FALSE);
  }

  /**
   * @param $component
   * @param $regex
   * @param bool $include
   */
  public function setComponentsByRegex($component, $regex, $include = TRUE) {
    if ($export_options = featurize_get_export_options($component)) {
      foreach ($export_options as $name => $value) {
        if (preg_match($regex, $name) > 0) {
          $this->setComponent($component, $name, $include);
        }
      }
    }
  }

  /**
   * @param $component
   * @param $regex
   */
  public function addComponentsByRegex($component, $regex) {
    $this->setComponentsByRegex($component, $regex, TRUE);
  }

  /**
   * @param $component
   * @param $regex
   */
  public function removeComponentsByRegex($component, $regex) {
    $this->setComponentsByRegex($component, $regex, FALSE);
  }

  /**
   * @param $component
   * @param bool $include
   */
  public function setAllComponents($component, $include = TRUE) {
    if ($export_options = featurize_get_export_options($component)) {
      foreach ($export_options as $name => $value) {
        $this->setComponent($component, $name, $include);
      }
    }
  }

  /**
   * @param $component
   */
  public function addAllComponents($component) {
    $this->setAllComponents($component, TRUE);
  }

  /**
   * @param $component
   */
  public function removeAllComponents($component) {
    $this->setAllComponents($component, FALSE);
  }
}
